import logging
import numpy as np
from crawlutils.text_utils import make_num

logger = logging.getLogger(__name__)


일억 = 100_000_000
일조 = 1_000_000_000_000


def format_number_100m(num):
    try:
        if num > 일조:
            return f"{int(num/일조):,}조 {int((num % 일조)/일억):,}억"
        elif num > 일억:
            return f"{int(num/일억):,}억"
        else:
            return f"{int(num):,}"
    except Exception as ex:
        logging.exception(ex)
        raise ValueError


def check_unit(check_text):
    if '조원' in check_text:
        unit = 1_000_000_000_000
    elif '억원' in check_text:
        unit = 100_000_000
    elif '백만원' in check_text:
        unit = 1_000_000
    elif '천원' in check_text:
        unit = 1_000
    elif '원' in check_text:
        unit = 1
    else:
        logging.error(f"check_text not contains unit string {check_text}")
        raise ValueError
    return unit


def defaultIfNumber(text, unit_multiplier, default="-"):
    op_num = make_num(text)
    op = default if op_num == '-' or op_num == '' else round(float(op_num) * unit_multiplier)
    return op

def to_int(text: str) -> int:
    if isinstance(text, int):
        return text
    try:
        t = text.replace(",", "").replace(" ", "")
        return np.nan if t == '' else int(t)
    except Exception as ex:
        logging.exception(ex)
        return np.nan

def to_float(text) -> float:
    try:
        return float(text.replace(",", "").replace(" ", ""))
    except Exception as ex:
        logging.exception(ex)
        return np.nan


import re


def make_text(t) -> str:
    return t.replace('\n', '').replace('\t', '').replace(' ', '').replace('\r', '').replace('\xa0', '')


def make_num(t) -> str:
    return t.replace('\n', '').replace('\t', '').replace(' ', '').replace('\r', '').replace('\xa0', '').replace(',', '')


def make_num_include_zo(txt):
    """
    ## 억원 이 생략된 억단위 금액을 숫자로 변환
    ## 1조 2,000 => 1_200_000_000_000
    :param txt:
    :return:
    """
    일조 = 1_000_000_000_000
    일억 = 100_000_000
    tkns = txt.split("조")
    zo_part = int(make_num(tkns[0])) * 일조 if len(tkns) > 1 else 0
    uk_part_txt = tkns[1] if len(tkns) > 1 else make_num(tkns[0])
    uk_part = int(make_num(uk_part_txt)) * 일억
    return zo_part + uk_part


def make_date(data):
    return data.replace('년', '-').replace('월', '-').replace('일', '').replace(' ', '').replace('.', '-')


def to_quarter_str(num):
    return f"{num // 100}.{(num % 100) // 3}"


def splitter(s):
    spl = s.split(" = ")
    if len(spl) <= 1:
        return None
    try:
        return int(spl[1])
    except ValueError:
        return None


def has_text_in_list(texts, str):
    for text in texts:
        if text in str:
            return True

    return False


def remove_strs(text, removes):
    tmp = text
    for remove in removes:
        tmp = tmp.replace(remove, '')

    return tmp

def trim(text):
    return text.strip() if text is not None else ""

def trim_template(text):
    return re.sub(r"^\s*\|", "", text.strip(), 0, re.MULTILINE) + "\n\n"


def json_encode(dictionary):
    return dictionary.encode()



import logging.config
import yaml


class FileLogger:
    def __init__(self, filename):
        self.filename = filename

        with open(filename, 'r') as stream:
            config = yaml.load(stream, Loader=yaml.FullLoader)

        logging.config.dictConfig(config)




import logging
from typing import Dict

import requests
from bs4 import BeautifulSoup
from cache import AdtCache, MemoryCache

from crawlutils.text_utils import make_num_include_zo, make_num

logger = logging.getLogger("naverweb")

naver_url = "https://finance.naver.com/item/main.nhn"


class NaverWeb:
    def __init__(self, cache: AdtCache = None):
        if cache is None:
            cache = MemoryCache()

        self.cache: AdtCache = cache

    def get_stock_info(self, code) -> Dict[str, int]:
        """
        :param code:
        :return:
         - market_cap : 시가총액
         - current_price : 현재가
         - total_stock_count: 상장주식수
        """
        if self.cache is not None:
            result = self.cache.hget(f"marketcap_{code}")
            if result is not None and len(result.items()) > 0:
                logger.debug(f"cache hit for market_cap {code} - {result}")
                for k, v in result.items():
                    result[k] = int(v)
                return result

        cp_r = requests.get(f"{naver_url}?code={code}")
        cp_soup = BeautifulSoup(cp_r.text, 'html.parser')

        price, mc, total_stock_count = -1, 0, 0

        try:
            price = int(cp_soup.findAll('dd')[3].text.split(' ')[1].replace(',', ''))
            mc = make_num_include_zo(cp_soup.findAll('em', id='_market_sum')[0].text)
            total_stock_count = int(make_num(cp_soup.find("th", string="상장주식수").parent.find("td").text))
            logger.debug(f"total_stock_count : {total_stock_count}")
        except Exception as ex:
            logger.exception(ex)

        data = {
            "market_cap": mc,
            "current_price": price,
            "total_stock_count": total_stock_count
        }

        if mc != 0:
            self.cache.hset(f"marketcap_{code}", data )
            logger.debug(f"set cache for code {code} - {data}")

        return data

    def get_cur_price(self, code) -> int:
        d = self.get_stock_info(code=code)
        return d.get("current_price")

    def get_market_cap(self, code) -> int:
        d = self.get_stock_info(code=code)
        return d.get("market_cap")


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG, format='[%(levelname)s] > %(message)s')
    naver = NaverWeb(cache=None)

    mc = naver.get_market_cap(code="000660")
    logger.info(mc)

    price = naver.get_cur_price(code="000660")
    logger.info(price)

    d = naver.get_stock_info(code="000660")
    logger.info(f"market cap : {mc}, price : {d.get('current_price')}, total_stock_count : {d.get('total_stock_count')}")


from datetime import datetime
import requests

class Interval:
    def __init__(self, level: int, fr: str, to: str, secs: int):
        self.level = level
        self.time_from = None if len(fr) < 1 else datetime.strptime(fr, "%H:%M").time()
        self.time_to = None if len(to) < 1 else datetime.strptime(to, "%H:%M").time()
        self.secs = secs

    def __str__(self):
        return f"level : {self.level}, from : {self.time_from}, to : {self.time_to}, secs : {self.time_to}"


class CrawlingInterval:
    """
    설정에 따라 crawling interval 을 설정한다
    """
    def __init__(self, intervals):
        self.intervals = list(map(lambda x: Interval(x.get("level"), x.get("from"), x.get("to"), x.get("secs")), intervals))

    def choose_sleep_time(self, dt):
        t = dt.time()
        default_secs = 10
        for interval in self.intervals:
            if interval.time_from is None:
                default_secs = interval.secs
            else:
                if interval.time_from <= t < interval.time_to:
                    return interval.secs

        return default_secs

def get_public_ip():
    headers = {
        'authority': 'icanhazip.com',
        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
        'accept-language': 'en-US,en;q=0.9,ko;q=0.8',
        'cache-control': 'no-cache',
        'pragma': 'no-cache',
        'sec-ch-ua': '"Chromium";v="112", "Google Chrome";v="112", "Not:A-Brand";v="99"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"macOS"',
        'sec-fetch-dest': 'document',
        'sec-fetch-mode': 'navigate',
        'sec-fetch-site': 'none',
        'sec-fetch-user': '?1',
        'upgrade-insecure-requests': '1',
        'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36',
    }

    response = requests.get('https://icanhazip.com/', headers=headers)
    return response.text.strip()


if __name__ == "__main__":
    # c = CrawlingInterval()
    # assert(0.4 == c.choose_sleep_time(datetime(1970, 1, 1, 10, 10)))
    # assert(1 == c.choose_sleep_time(datetime(1970, 1, 1, 16, 10)))
    # assert(1 == c.choose_sleep_time(datetime(1970, 1, 1, 8, 59)))
    # assert(0.4 == c.choose_sleep_time(datetime(1970, 1, 1, 15, 59)))
    # assert(0.4 == c.choose_sleep_time(datetime(1970, 1, 1, 9, 0)))
    #
    ip = get_public_ip()
    print(f"public ip : [{ip}]")


from crawlutils.num_utils import defaultIfNumber


def get_td(td_list, *text_in):
    for i in td_list:
        valid = True
        for text in text_in:
            if text not in i.text.replace(" ", ""):
                valid = False
                break
        if valid:
            return i
    return "-"


def get_td_text(td_list, *text_in):
    for i in td_list:
        valid = True
        for text in text_in:
            if text not in i.text.replace(" ", ""):
                valid = False
                break
        if valid:
            return i.findNext('td').text.strip()
    return "-"


def find_tr_has_text(text, trlist):
    target_tr = None
    for tr in trlist:
        if text in tr.text.replace(' ', ''):
            target_tr = tr
            break
    return target_tr


def defaultNumberInTR(text, tr3, unit_multiplier, default="-"):
    np_tr = find_tr_has_text(text, tr3)
    np = defaultIfNumber(np_tr.findAll('td')[1].text, unit_multiplier, default=default)
    return np